from ..filehandling.constants import *
NODE = "Node"
NODES = "Nodes"
IS_ROOT = "root"
AGGREGATES_TO = "aggregates_to"
SCRIPTS = "Scripts"
SCRIPT = "Script"
IS_CONCEPT = "is_concept"
IS_IDENTIFIER = "is_identifier"
NONE = "None"
IDENTIFIER = "identifer"
AGGREGATE = "Aggregate"
ATTRIBUTE = "Attribute"
PROPERTY_SET = "PropertySet"
PREDEFINED_PSET = "PredefinedPropertySet"
PREDEFINED_PROPERTY_WINDOW_NAME = "Predefined Properties"
INHERITED_TEXT = "Predefined Pset"
